//	Dan's Color Converter - actual main class implementation
//	
//	Copyright 1993 by Daniel Lyke
//
//
//	This is EgoWare. You may freely copy and distribute these programs as long
//	as all parts are distributed together and unaltered. All source code may be
//	incorporated into your own programs, but I'd appreciate it if you drop me a
//	postcard or message telling me that you're using it and what you're using
//	it for. I make no warranties, express or implied, concerning the proper
//	operation of the program or any of the accompanying source code.

#include <math.h>
#include "color.h"

void TBigColor::GetRGB(double &rdRed, double &rdGreen, double &rdBlue)
{
	rdRed		=	mdRed;
	rdGreen	=	mdGreen;
	rdBlue	=	mdBlue;
}
void TBigColor::SetRGB(double dRed, double dGreen, double dBlue)
{
	mdRed		=	dRed;
	mdGreen	=	dGreen;
	mdBlue	=	dBlue;
}

void TBigColor::GetCMY(double &rdCyan, double &rdMagenta, double &rdYellow)
{
	rdCyan		=	1.0 - mdRed;
	rdMagenta	=	1.0 - mdGreen;
	rdYellow	=	1.0 - mdBlue;
}
void TBigColor::SetCMY(double dCyan, double dMagenta, double dYellow)
{
	mdRed		=	1.0 - dCyan;
	mdGreen	=	1.0 - dMagenta;
	mdBlue	=	1.0 - dYellow;
}




void TBigColor::GetHLS(double &rdHue, double &rdLuminance, double &rdSaturation)
{
	double dMin;
	double dMax;
	double dDelta;

	// Find the minimum of the RGB triplet
	dMin = mdRed;
	if (mdGreen < dMin)
		dMin = mdGreen;
	if (mdBlue < dMin)
		dMin = mdBlue;

	// Find the maximum of the RGB triplet
	dMax = mdRed;
	if (mdGreen > dMax)
		dMax = mdGreen;
	if (mdBlue > dMax)
		dMax = mdBlue;

	rdLuminance = (dMax + dMin) / 2.0;

	if (dMax == dMin)
		{
		// achromatic case

		rdSaturation = 0.0;
		rdHue = HUE_UNDEFINED;
		}
	else
		{
		dDelta = dMax - dMin;

		if (rdLuminance <= 0.5)
			{
			rdSaturation = dDelta / (dMax + dMin);
			}
		else
			{
			rdSaturation = dDelta / (2.0 - dMax - dMin);
			}

		if (mdRed == dMax)
			{
			rdHue = (mdGreen - mdBlue) / dDelta;
			}
		else if (mdGreen == dMax)
			{
			rdHue = 2.0 + (mdBlue - mdRed) / dDelta;
			}
		else // mdBlue == dMax
			{
			rdHue = 4.0 + (mdRed - mdGreen) / dDelta;
			}
		rdHue = rdHue * 60.0;

		while (rdHue < 0.0)
			{
			rdHue += 360.0;
			}
		}

	rdHue /= 360.0;

} // end TBigColor::GetHLS

double TBigColor::SetHLSValue(double dM1, double dM2, double dHue)
{
	while (dHue < 0.0)
		{
		dHue += 360.0;
		}
	while (dHue >= 360.0)
		{
		dHue -= 360.0;
		}

	if (dHue < 60.0)
		{
		return dM1 + (dM2 - dM1) * dHue / 60.0;
		}
	else if (dHue < 180.)
		{
		return dM2;
		}
	else if (dHue < 240.0)
		{
		return dM1 + (dM2 - dM1) * (240.0 - dHue) / 60.0;
		}

	return dM1;
} // end TBigColor::SetHLSValue



void TBigColor::SetHLS(double dHue, double dLuminance, double dSaturation)
{
	double dM1;
	double dM2;


	if (dLuminance <= 0.5)
		{
		dM2 = dLuminance * (1.0 + dSaturation);
		}
	else
		{
		dM2 = dLuminance + dSaturation - dLuminance * dSaturation;
		}

	dM1 = 2.0 * dLuminance - dM2;


	if (dSaturation == 0.0)
		{
		mdRed		=	dLuminance;
		mdGreen	=	dLuminance;
		mdBlue	=	dLuminance;
		}
	else
		{
		dHue		*=	360.0;
		mdRed		=	SetHLSValue(dM1, dM2, dHue + 120.0);
		mdGreen	=	SetHLSValue(dM1, dM2, dHue);
		mdBlue	=	SetHLSValue(dM1, dM2, dHue - 120.0);
		}

} // end TBigColor::SetHLS





void TBigColor::GetHSV(double &rdHue, double &rdSaturation, double &rdValue)
{
	double dMin;
	double dMax;
	double dDelta;

	// Find the minimum of the RGB triplet
	dMin = mdRed;
	if (mdGreen < dMin)
		dMin = mdGreen;
	if (mdBlue < dMin)
		dMin = mdBlue;

	// Find the maximum of the RGB triplet
	dMax = mdRed;
	if (mdGreen > dMax)
		dMax = mdGreen;
	if (mdBlue > dMax)
		dMax = mdBlue;


	rdValue = dMax;

	if (dMax != 0)
		{
		rdSaturation = (dMax - dMin) / dMax;

		dDelta = dMax - dMin;

		if (mdRed == dMax)
			{
			rdHue = 0.0 + (mdGreen - mdBlue) / dDelta;
			}
		else if (mdGreen == dMax)
			{
			rdHue = 2.0 + (mdBlue - mdRed) / dDelta;
			}
		else // mdBlue == dMax
			{
			rdHue = 4.0 + (mdRed - mdGreen) / dDelta;
			}

		rdHue = rdHue * 60.0;

		while (rdHue < 0.0)
			{
			rdHue += 360.0;
			}
		while (rdHue >= 360.0)
			{
			rdHue -= 360.0;
			}


		while (rdHue < 0.0)
			{
			rdHue += 360.0;
			}
		while (rdHue >= 360.0)
			{
			rdHue -= 360.0;
			}

		rdHue /= 360.0;
		}
	else
		{
		rdSaturation = 0.0;
		rdHue = HUE_UNDEFINED;
		}
} // end TBigColor::GetHSV



void TBigColor::SetHSV(double dHue, double dSaturation, double dValue)
{
	dHue *= 360.0;

	while (dHue < 0.0)
		{
		dHue += 360.0;
		}
	while (dHue >= 360.0)
		{
		dHue -= 360.0;
		}


	if (dSaturation == 0.0)
		{
		mdRed		= dValue;
		mdGreen	= dValue;
		mdBlue	= dValue;
		}
	else
		{
		dHue /= 60.0;

		double dFract;
		double dP;
		double dQ;
		double dT;

		dFract = dHue - floor(dHue);


		dP = dValue * (1.0 - dSaturation);
		dQ = dValue * (1.0 - (dSaturation * (dHue - floor(dHue))));
		dT = dValue * (1.0 - (dSaturation * (ceil(dHue) - dHue)));
		
		switch ((int)floor(dHue))
			{
			case 0 :
				mdRed		= dValue;
				mdGreen	= dT;
				mdBlue	= dP;
				break;
			case 1 :
				mdRed		= dQ;
				mdGreen	= dValue;
				mdBlue	= dP;
				break;
			case 2 :
				mdRed		= dP;
				mdGreen	= dValue;
				mdBlue	= dT;
				break;
			case 3 :
				mdRed		= dP;
				mdGreen	= dQ;
				mdBlue	= dValue;
				break;
			case 4 :
				mdRed		= dT;
				mdGreen	= dP;
				mdBlue	= dValue;
				break;
			case 5 :
				mdRed		= dValue;
				mdGreen	= dP;
				mdBlue	= dQ;
				break;

			default :
				mdRed		= 0.0;
				mdGreen	= 0.0;
				mdBlue	= 0.0;
				break;
			}
		}
} // end TBigColor::SetHSV




void TBigColor::GetYIQ(double &rdY, double &rdI, double &rdQ)
{
	rdY	= .30 * mdRed +  .59 * mdGreen +  .11 * mdBlue;
	rdI	= .60 * mdRed + -.28 * mdGreen + -.32 * mdBlue;
	rdQ	= .21 * mdRed + -.52 * mdGreen +  .31 * mdBlue;
}



void TBigColor::SetYIQ(double dY, double dI, double dQ)
{
	mdRed		= 1.000000 * dY + 0.948262  * dI + 0.624013  * dQ;
	mdGreen	= 1.000000 * dY + -0.276066 * dI + -0.639810	* dQ;
	mdBlue	= 1.000000 * dY + -1.105450 * dI + 1.729858 	* dQ;
}



void TBigColor::GetYUV(double &rdY, double &rdU, double &rdV)
{
	rdY	=  0.299 * mdRed +	 0.587 * mdGreen +	 0.114 * mdBlue;
	rdU	= -0.147 * mdRed +	-0.289 * mdGreen +	 0.437 * mdBlue;
	rdV	=  0.615 * mdRed +	-0.515 * mdGreen +	-0.100 * mdBlue;
}



void TBigColor::SetYUV(double dY, double dU, double dV)
{
	mdRed		= 1.0 * dY +	 0.0	 * dU + 	 1.140 * dV;
	mdGreen	= 1.0 * dY +	-0.394 * dU + 	-0.581 * dV;
	mdBlue	= 1.0 * dY +	 2.028 * dU + 	 0.0	 * dV;
}




void TBigColor::Get(int iType, double &d1, double &d2, double &d3)
{
	switch(iType)
		{
		case COLORTYPE_RGB :
			GetRGB(d1,d2,d3);
			break;

		case COLORTYPE_CMY :
			GetCMY(d1,d2,d3);
			break;

		case COLORTYPE_HLS :
			GetHLS(d1,d2,d3);
			break;

		case COLORTYPE_HSV :
			GetHSV(d1,d2,d3);
			break;

		case COLORTYPE_YIQ :
			GetYIQ(d1,d2,d3);
			break;

		case COLORTYPE_YUV :
			GetYUV(d1,d2,d3);
			break;

		default :
			d1 = d2 = d3 = 0.0;
			break;
		} // end of switch on type
}

void TBigColor::Set(int iType, double d1, double d2, double d3)
{
	switch(iType)
		{
		case COLORTYPE_RGB :
			SetRGB(d1,d2,d3);
			break;

		case COLORTYPE_CMY :
			SetCMY(d1,d2,d3);
			break;

		case COLORTYPE_HLS :
			SetHLS(d1,d2,d3);
			break;

		case COLORTYPE_HSV :
			SetHSV(d1,d2,d3);
			break;

		case COLORTYPE_YIQ :
			SetYIQ(d1,d2,d3);
			break;

		case COLORTYPE_YUV :
			SetYUV(d1,d2,d3);
			break;

		default :
			d1 = d2 = d3 = 0.0;
			break;
		} // end of switch on type
}

