
//	Dan's Color Converter - source code for the DOS version
//	
//	Copyright 1993 by Daniel Lyke
//
//
//	This is EgoWare. You may freely copy and distribute these programs as long
//	as all parts are distributed together and unaltered. All source code may be
//	incorporated into your own programs, but I'd appreciate it if you drop me a
//	postcard or message telling me that you're using it and what you're using
//	it for. I make no warranties, express or implied, concerning the proper
//	operation of the program or any of the accompanying source code.




#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <math.h>
#include "color.h"


static char gaachConversionTypes[][4] =
{
	"RGB", // COLORTYPE_RGB
	"CMY", // COLORTYPE_CMY
	"HLS", // COLORTYPE_HLS
	"HSV", // COLORTYPE_HSV
	"YIQ", // COLORTYPE_YIQ
	"YUV", // COLORTYPE_YUV

	"\0\0\0"
};




#define STATE_INVALID		-1
#define STATE_INIT			0
#define STATE_COLOR1			1
#define STATE_COLOR2			2
#define STATE_COLOR3			3
#define STATE_OUTPUT			4
#define STATE_GETINSCALE	5
#define STATE_GETOUTSCALE	6



char *gapszContextHelp[] =
{
	// 0
	"  Expecting an input color type (\"RGB\", \"CMY\", \"HLS\", \"HSV\", \n"
	"  \"YIQ\", \"YUV\"), \"SCALE\", \"DEGREES\" or \"RADIANS\", got\n"
	"  %s instead at argument %d.",

	// 1
	"  Expecting a first color value, got %s instead at argument %d.",
	// 2
	"  Expecting a second color value, got %s instead at argument %d.",
	// 3
	"  Expecting a third color value, got %s instead at argument %d.",

	// 4
	"  Expecting an output color type (\"RGB\", \"CMY\", \"HLS\", \"HSV\",\n"
	"  \"YIQ\", \"YUV\") or \"SCALE\", got %s instead at argument %d.",

	// 5
	"  Expecting a value to scale the input colors (except for \"Hue\" in\n"
	"  HLS and HSV models) by, usually 255. Default is 1.",

	"  Expecting a value to scale the output colors (except for \"Hue\" in\n"
	"  HLS and HSV models) by, usually 255. Default is 1.",
};


char *gpszGenericHelp =
	"Dan's Color Converter - Revision 0.6 Written 9/22/93 by Daniel Lyke\n"
	"\n"
	"This is not yet a finished product, bear with me. See the documentation\n"
	"for a further discussion, or contact me at:\n"
	"\n"
	"   3405 A Audubon Drive            FidoNet: 1:362/1203\n"
	"   Chattanooga, TN 37411-4402      BBS: 615/493-9826\n"
	"\n"
	"\n"
	"Usage:\n"
	"   COLORCVT [SCALE <in scale>] [<angle units>] <in type> <color1> <color2> <color3>\n"
	"            [SCALE <out scale>] [<angle units>] <out type>\n"
	"\n"
	"	where <in type> and <out type> are one of RGB, CMY, HLS, HSV or YIQ,\n"
	"	<color1-3> are 0 - <in scale> or a valid angle number for hue components.\n"
	"\n"
	"	Valid angle numbers are determined by <angle units>, either\n"
	"	\"DEGREES\" or \"RADIANS\".  Default is the respective input or\n"
	"	output scale.\n"
	"\n"
	"Examples:\n"
	"	COLORCVT RGB .5 .3 .7 YIQ\n"
	"	COLORCVT SCALE 256 RGB .7 .34 .2 SCALE 100 CMY\n";


#define STATE_INIT			0
#define STATE_COLOR1			1
#define STATE_COLOR2			2
#define STATE_COLOR3			3
#define STATE_OUTPUT			4
#define STATE_GETINSCALE	5
#define STATE_GETOUTSCALE	6


int main(int iArgc, char **ppszArgv)
{
	TBigColor color;
	int iArg;
	int iState;
	int iError;
	int iIndex;
	int iSourceType;
	int iTargetType;
	double dColor1;
	double dColor2;
	double dColor3;
	double dOut1;
	double dOut2;
	double dOut3;
	double dInScale;
	double dOutScale;
	double dInAngleUnits;
	double dOutAngleUnits;



	iArg = 1;
	iState = STATE_INIT;

	dInScale = 1.0;
	dOutScale = 1.0;
	dInAngleUnits = 0.0;
	dOutAngleUnits = 0.0;

	iError = 0;

	while (iArg < iArgc && !iError)
		{
		switch (iState)
			{
			case STATE_INIT :
				iSourceType = -1;

				for (iIndex = 0; gaachConversionTypes[iIndex][0]; iIndex++)
					{
					if (!stricmp(ppszArgv[iArg],gaachConversionTypes[iIndex]))
						{
						iSourceType = iIndex;
						break;
						}
					} // end of looping through the types

				if (-1 == iSourceType)
					{
					if (!stricmp(ppszArgv[iArg],"SCALE"))
						{
						iState = STATE_GETINSCALE;
						}
					else if (!stricmp(ppszArgv[iArg],"DEGREES"))
						{
						dInAngleUnits = 360.0;
						}
					else if (!stricmp(ppszArgv[iArg],"RADIANS"))
						{
						dInAngleUnits = M_PI * 2.0;
						}
					else
						{
						iError = 1;
						}
					}
				else
					{
					iState = STATE_COLOR1;
					}
				break;

			case STATE_COLOR1 :
				if (isdigit(ppszArgv[iArg][0]))
					{
					dColor1 = atof(ppszArgv[iArg]);
					iState = STATE_COLOR2;
					}
				else
					{
					iError = 1;
					}
				break;

			case STATE_COLOR2 :
				if (isdigit(ppszArgv[iArg][0]))
					{
					dColor2 = atof(ppszArgv[iArg]);
					iState = STATE_COLOR3;
					}
				else
					{
					iError = 1;
					}
				break;

			case STATE_COLOR3 :
				if (isdigit(ppszArgv[iArg][0]))
					{
					dColor3 = atof(ppszArgv[iArg]);
					iState = 4;
					}
				else
					{
					iError = 1;
					}
				break;

			case STATE_OUTPUT :
				iTargetType = -1;

				for (iIndex = 0; gaachConversionTypes[iIndex][0]; iIndex++)
					{
					if (!stricmp(ppszArgv[iArg],gaachConversionTypes[iIndex]))
						{
						iTargetType = iIndex;
						break;
						}
					} // end of looping through the types

				if (-1 == iTargetType)
					{
					if (!stricmp(ppszArgv[iArg],"SCALE"))
						{
						iState = STATE_GETOUTSCALE;
						}
					else if (!stricmp(ppszArgv[iArg],"DEGREES"))
						{
						dOutAngleUnits = 360.0;
						}
					else if (!stricmp(ppszArgv[iArg],"RADIANS"))
						{
						dOutAngleUnits = M_PI * 2.0;
						}
					else
						{
						iError = 1;
						break;
						}
					}
				else
					{

					if (0.0 == dInAngleUnits)
						{
						dInAngleUnits = dInScale;
						}

					if (0.0 == dOutAngleUnits)
						{
						dOutAngleUnits = dOutScale;
						}



					if (iSourceType == COLORTYPE_HLS ||
						iSourceType == COLORTYPE_HSV)
						{
						color.Set(iSourceType, 
							dColor1 / dInAngleUnits,
							dColor2 / dInScale,
							dColor3 / dInScale );
						}
					else
						{
						color.Set(iSourceType,
							dColor1 / dInScale, 
							dColor2 / dInScale,
							dColor3 / dInScale );
						}

					color.Get(iTargetType, dOut1, dOut2, dOut3);

					if (iTargetType == COLORTYPE_HLS ||
						iTargetType == COLORTYPE_HSV)
						{
						dOut1 *= dOutAngleUnits;
						dOut2 *= dOutScale;
						dOut3 *= dOutScale;
						}
					else
						{
						dOut1 *= dOutScale;
						dOut2 *= dOutScale;
						dOut3 *= dOutScale;
						}

					printf("%s (%.4f, %.4f, %.4f) to %s (%.4f, %.4f, %.4f) scale %3.2f\n",
						gaachConversionTypes[iSourceType],
						dColor1,
						dColor2,
						dColor3,
						gaachConversionTypes[iTargetType],
						dOut1,
						dOut2,
						dOut3,
						dOutScale);

					iState = STATE_INIT;
					}
				break;


			case STATE_GETINSCALE :
				if (isdigit(ppszArgv[iArg][0]))
					{
					dInScale = atof(ppszArgv[iArg]);
					iState = STATE_INIT;
					}
				else
					{
					iError = 1;
					}
				break;

			case STATE_GETOUTSCALE :
				if (isdigit(ppszArgv[iArg][0]))
					{
					dOutScale = atof(ppszArgv[iArg]);
					iState = STATE_OUTPUT;
					}
				else
					{
					iError = 1;
					}
				break;
			} // end of switching on the state

		iArg++;
		} // end of looking through the arguments

	if (iState != STATE_INIT)
		{
		printf("Dan's Color Converter: usage error:\n");
		printf(gapszContextHelp[iState], ppszArgv[iArg - 1], iArg - 1);
		return 1;
		}
	else if (iArgc <= 1)
		{
		printf("%s", gpszGenericHelp);
		return 1;
		}

	return 0;
}
